@@ -37,14 +37,15 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
+#include <libpeasui/peas-ui-plugin-manager.h>
 
 #include "gedit-preferences-dialog.h"
 #include "gedit-utils.h"
 #include "gedit-debug.h"
 #include "gedit-document.h"
 #include "gedit-style-scheme-manager.h"
-#include "gedit-plugin-manager.h"
 #include "gedit-dirs.h"
+#include "gedit-plugins-engine.h"
 #include "gedit-settings.h"
 #include "gedit-utils.h"
 
@@ -289,9 +290,9 @@ wrap_mode_checkbutton_toggled (GtkToggleButton        *button,
 		}
 	}
 
-	gedit_settings_set_wrap_mode (dlg->priv->editor,
-				      GEDIT_SETTINGS_WRAP_MODE,
-				      mode);
+	g_settings_set_enum (dlg->priv->editor,
+			     GEDIT_SETTINGS_WRAP_MODE,
+			     mode);
 }
 
 static void
@@ -327,8 +328,8 @@ setup_view_page (GeditPreferencesDialog *dlg)
 			"u", &right_margin_position);
 	
 	/* Set initial state */
-	wrap_mode = gedit_settings_get_wrap_mode (dlg->priv->editor,
-						  GEDIT_SETTINGS_WRAP_MODE);
+	wrap_mode = g_settings_get_enum (dlg->priv->editor,
+					 GEDIT_SETTINGS_WRAP_MODE);
 
 	switch (wrap_mode)
 	{
@@ -903,11 +904,14 @@ setup_font_colors_page (GeditPreferencesDialog *dlg)
 static void
 setup_plugins_page (GeditPreferencesDialog *dlg)
 {
+	GeditPluginsEngine *engine;
 	GtkWidget *page_content;
 
 	gedit_debug (DEBUG_PREFS);
 
-	page_content = gedit_plugin_manager_new ();
+	engine = gedit_plugins_engine_get_default ();
+
+	page_content = peas_ui_plugin_manager_new (PEAS_ENGINE (engine));
 	g_return_if_fail (page_content != NULL);
 
 	gtk_box_pack_start (GTK_BOX (dlg->priv->plugin_manager_place_holder),
