@@ -44,7 +44,6 @@
 #include "gedit-utils.h"
 #include "gedit-enum-types.h"
 #include "gedit-dirs.h"
-#include "gseal-gtk-compat.h"
 #include "gedit-settings.h"
 
 #ifdef OS_OSX
@@ -277,7 +276,7 @@ gedit_app_class_init (GeditAppClass *klass)
 static gboolean
 ensure_user_config_dir (void)
 {
-	gchar *config_dir;
+	const gchar *config_dir;
 	gboolean ret = TRUE;
 	gint res;
 
@@ -295,43 +294,39 @@ ensure_user_config_dir (void)
 		ret = FALSE;
 	}
 
-	g_free (config_dir);
-
 	return ret;
 }
 
 static void
 load_accels (void)
 {
-	gchar *filename;
+	const gchar *filename;
 
 	filename = gedit_dirs_get_user_accels_file ();
 	if (filename != NULL)
 	{
 		gedit_debug_message (DEBUG_APP, "Loading keybindings from %s\n", filename);
 		gtk_accel_map_load (filename);
-		g_free (filename);
 	}
 }
 
 static void
 save_accels (void)
 {
-	gchar *filename;
+	const gchar *filename;
 
 	filename = gedit_dirs_get_user_accels_file ();
 	if (filename != NULL)
 	{
 		gedit_debug_message (DEBUG_APP, "Saving keybindings in %s\n", filename);
 		gtk_accel_map_save (filename);
-		g_free (filename);
 	}
 }
 
 static gchar *
 get_page_setup_file (void)
 {
-	gchar *config_dir;
+	const gchar *config_dir;
 	gchar *setup = NULL;
 
 	config_dir = gedit_dirs_get_user_config_dir ();
@@ -341,7 +336,6 @@ get_page_setup_file (void)
 		setup = g_build_filename (config_dir,
 					  GEDIT_PAGE_SETUP_FILE,
 					  NULL);
-		g_free (config_dir);
 	}
 
 	return setup;
@@ -404,7 +398,7 @@ save_page_setup (GeditApp *app)
 static gchar *
 get_print_settings_file (void)
 {
-	gchar *config_dir;
+	const gchar *config_dir;
 	gchar *settings = NULL;
 
 	config_dir = gedit_dirs_get_user_config_dir ();
@@ -414,7 +408,6 @@ get_print_settings_file (void)
 		settings = g_build_filename (config_dir,
 					     GEDIT_PRINT_SETTINGS_FILE,
 					     NULL);
-		g_free (config_dir);
 	}
 
 	return settings;
@@ -712,6 +705,7 @@ gedit_app_create_window_real (GeditApp    *app,
 /**
  * gedit_app_create_window:
  * @app: the #GeditApp
+ * @screen: (allow-none):
  *
  * Create a new #GeditWindow part of @app.
  *
@@ -752,8 +746,8 @@ _gedit_app_restore_window (GeditApp    *app,
  *
  * Returns all the windows currently present in #GeditApp.
  *
- * Return value: the list of #GeditWindows objects. The list
- * should not be freed
+ * Return value: (element-type Window) (transfer none): the list of #GeditWindows objects.
+ * The list should not be freed
  */
 const GList *
 gedit_app_get_windows (GeditApp *app)
@@ -895,7 +889,8 @@ _gedit_app_get_window_in_viewport (GeditApp  *app,
  *
  * Returns all the documents currently open in #GeditApp.
  *
- * Return value: a newly allocated list of #GeditDocument objects
+ * Return value: (element-type Gedit.Document) (transfer container):
+ * a newly allocated list of #GeditDocument objects
  */
 GList *
 gedit_app_get_documents	(GeditApp *app)
@@ -924,7 +919,8 @@ gedit_app_get_documents	(GeditApp *app)
  *
  * Returns all the views currently present in #GeditApp.
  *
- * Return value: a newly allocated list of #GeditView objects
+ * Return value: (element-type Gedit.View) (transfer container):
+ * a newly allocated list of #GeditView objects
  */
 GList *
 gedit_app_get_views (GeditApp *app)
