@@ -39,6 +39,7 @@
 #include "gedit-commands.h"
 #include "gedit-window.h"
 #include "gedit-notebook.h"
+#include "gedit-multi-notebook.h"
 #include "gedit-debug.h"
 
 void
@@ -69,7 +70,6 @@ void
 _gedit_cmd_documents_move_to_new_window (GtkAction   *action,
 					 GeditWindow *window)
 {
-	GeditNotebook *old_notebook;
 	GeditTab *tab;
 
 	gedit_debug (DEBUG_COMMANDS);
@@ -79,11 +79,29 @@ _gedit_cmd_documents_move_to_new_window (GtkAction   *action,
 	if (tab == NULL)
 		return;
 
-	old_notebook = GEDIT_NOTEBOOK (_gedit_window_get_notebook (window));
-
-	g_return_if_fail (gtk_notebook_get_n_pages (GTK_NOTEBOOK (old_notebook)) > 1);
-
 	_gedit_window_move_tab_to_new_window (window, tab);
 }
 
+/* Methods releated with the tab groups */
+void
+_gedit_cmd_documents_new_tab_group (GtkAction   *action,
+				    GeditWindow *window)
+{
+	gedit_multi_notebook_add_new_notebook (GEDIT_MULTI_NOTEBOOK (_gedit_window_get_multi_notebook (window)));
+}
+
+void
+_gedit_cmd_documents_previous_tab_group (GtkAction   *action,
+					 GeditWindow *window)
+{
+	gedit_multi_notebook_previous_notebook (GEDIT_MULTI_NOTEBOOK (_gedit_window_get_multi_notebook (window)));
+}
+
+void
+_gedit_cmd_documents_next_tab_group (GtkAction   *action,
+				     GeditWindow *window)
+{
+	gedit_multi_notebook_next_notebook (GEDIT_MULTI_NOTEBOOK (_gedit_window_get_multi_notebook (window)));
+}
+
 /* ex:ts=8:noet: */
