@@ -51,8 +51,6 @@
 #include "gedit-enum-types.h"
 #include "gedit-settings.h"
 
-#include "gseal-gtk-compat.h"
-
 #define AFTER_ALL_TABS -1
 #define NOT_IN_APP_WINDOWS -2
 
@@ -211,8 +209,6 @@ gedit_notebook_grab_focus (GtkWidget *widget)
 	GtkWidget *tab;
 	gint current_page;
 
-	GTK_WIDGET_CLASS (gedit_notebook_parent_class)->grab_focus (widget);
-
 	current_page = gtk_notebook_get_current_page (nb);
 	tab = gtk_notebook_get_nth_page (nb, current_page);
 
@@ -276,13 +272,33 @@ gedit_notebook_class_init (GeditNotebookClass *klass)
 	g_type_class_add_private (object_class, sizeof (GeditNotebookPrivate));
 }
 
+static GeditNotebook *
+get_notebook_from_widget (GtkWidget *widget)
+{
+	if (widget == NULL)
+	{
+		return NULL;
+	}
+
+	while (!gtk_widget_is_toplevel (GTK_WIDGET (widget)))
+	{
+		if (GEDIT_IS_NOTEBOOK (widget))
+		{
+			return GEDIT_NOTEBOOK (widget);
+		}
+
+		widget = gtk_widget_get_parent (GTK_WIDGET (widget));
+	}
+
+	return NULL;
+}
+
 static GeditNotebook *
 find_notebook_at_pointer (gint abs_x,
 			  gint abs_y)
 {
 	GdkWindow *win_at_pointer;
-	GdkWindow *toplevel_win;
-	gpointer toplevel = NULL;
+	gpointer widget = NULL;
 	gint x, y;
 
 	/* FIXME multi-head */
@@ -293,21 +309,10 @@ find_notebook_at_pointer (gint abs_x,
 		return NULL;
 	}
 
-	toplevel_win = gdk_window_get_toplevel (win_at_pointer);
-
 	/* get the GtkWidget which owns the toplevel GdkWindow */
-	gdk_window_get_user_data (toplevel_win, &toplevel);
+	gdk_window_get_user_data (win_at_pointer, &widget);
 
-	/* toplevel should be an GeditWindow */
-	if ((toplevel != NULL) && 
-	    GEDIT_IS_WINDOW (toplevel))
-	{
-		return GEDIT_NOTEBOOK (_gedit_window_get_notebook
-						(GEDIT_WINDOW (toplevel)));
-	}
-
-	/* We are outside all windows containing a notebook */
-	return NULL;
+	return get_notebook_from_widget (widget);
 }
 
 static gboolean
@@ -427,7 +432,7 @@ gedit_notebook_move_tab (GeditNotebook *src,
 			 GeditTab      *tab,
 			 gint           dest_position)
 {
-	g_return_if_fail (GEDIT_IS_NOTEBOOK (src));	
+	g_return_if_fail (GEDIT_IS_NOTEBOOK (src));
 	g_return_if_fail (GEDIT_IS_NOTEBOOK (dest));
 	g_return_if_fail (src != dest);
 	g_return_if_fail (GEDIT_IS_TAB (tab));
@@ -768,60 +773,6 @@ gedit_notebook_switch_page_cb (GtkNotebook     *notebook,
 	gtk_widget_grab_focus (child);
 }
 
-static gboolean
-show_tabs_mode_get_mapping (GValue   *value,
-			    GVariant *variant,
-			    gpointer  user_data G_GNUC_UNUSED)
-{
-	const gchar *str;
-	GeditNotebookShowTabsModeType show_tabs_mode;
-
-	str = g_variant_get_string (variant, NULL);
-
-	if (g_strcmp0 (str, "NEVER") == 0)
-	{
-		show_tabs_mode = GEDIT_NOTEBOOK_SHOW_TABS_NEVER;
-	}
-	else if (g_strcmp0 (str, "AUTO") == 0)
-	{
-		show_tabs_mode = GEDIT_NOTEBOOK_SHOW_TABS_AUTO;
-	}
-	else /* ALWAYS */
-	{
-		show_tabs_mode = GEDIT_NOTEBOOK_SHOW_TABS_ALWAYS;
-	}
-	
-	g_value_set_enum (value, show_tabs_mode);
-
-	return TRUE;
-}
-
-static GVariant *
-show_tabs_mode_set_mapping (const GValue       *value,
-			    const GVariantType *expected_type,
-			    gpointer            user_data G_GNUC_UNUSED)
-{
-	GVariant *result = NULL;
-
-	switch (g_value_get_enum (value))
-	{
-		case GEDIT_NOTEBOOK_SHOW_TABS_NEVER:
-			result = g_variant_new_string ("NEVER");
-			break;
-
-		case GEDIT_NOTEBOOK_SHOW_TABS_AUTO:
-			result = g_variant_new_string ("AUTO");
-			break;
-
-		case GEDIT_NOTEBOOK_SHOW_TABS_ALWAYS:
-		default:
-			result = g_variant_new_string ("ALWAYS");
-			break;
-	}
-
-	return result;
-}
-
 static void
 gedit_notebook_init (GeditNotebook *notebook)
 {
@@ -837,14 +788,11 @@ gedit_notebook_init (GeditNotebook *notebook)
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), TRUE);
 
-	g_settings_bind_with_mapping (notebook->priv->ui_settings,
-				      GEDIT_SETTINGS_SHOW_TABS_MODE,
-				      notebook,
-				      "show-tabs-mode",
-				      G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET,
-				      show_tabs_mode_get_mapping,
-				      show_tabs_mode_set_mapping,
-				      NULL, NULL);
+	g_settings_bind (notebook->priv->ui_settings,
+			 GEDIT_SETTINGS_SHOW_TABS_MODE,
+			 notebook,
+			 "show-tabs-mode",
+			 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
 
 	g_signal_connect (notebook,
 			  "button-press-event",
