@@ -21,7 +21,8 @@
  */
 
 #include "gedit-status-combo-box.h"
-#include "gseal-gtk-compat.h"
+
+#include <gdk/gdkkeysyms.h>
 
 #define COMBO_BOX_TEXT_DATA "GeditStatusComboBoxTextData"
 
@@ -195,9 +196,9 @@ menu_position_func (GtkMenu		*menu,
 }
 
 static void
-button_press_event (GtkWidget           *widget,
-		    GdkEventButton      *event,
-		    GeditStatusComboBox *combo)
+show_menu (GeditStatusComboBox *combo,
+	   guint                button,
+	   guint32              time)
 {
 	GtkRequisition request;
 	gint max_height;
@@ -215,13 +216,13 @@ button_press_event (GtkWidget           *widget,
 		gtk_widget_set_size_request (gtk_widget_get_toplevel (combo->priv->menu), -1, max_height);
 	}
 	
-	gtk_menu_popup (GTK_MENU (combo->priv->menu), 
-			NULL, 
-			NULL, 
-			(GtkMenuPositionFunc)menu_position_func, 
-			combo, 
-			event->button, 
-			event->time);
+	gtk_menu_popup (GTK_MENU (combo->priv->menu),
+			NULL,
+			NULL,
+			(GtkMenuPositionFunc)menu_position_func,
+			combo,
+			button,
+			time);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (combo->priv->button), TRUE);
 
@@ -232,6 +233,36 @@ button_press_event (GtkWidget           *widget,
 	}
 }
 
+static gboolean
+button_press_event (GtkWidget           *widget,
+		    GdkEventButton      *event,
+		    GeditStatusComboBox *combo)
+{
+	if (event->type == GDK_BUTTON_PRESS && event->button == 1)
+	{
+		show_menu (combo, event->button,event->time);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+key_press_event (GtkWidget           *widget,
+		 GdkEventKey         *event,
+		 GeditStatusComboBox *combo)
+{
+	if (event->keyval == GDK_Return || event->keyval == GDK_ISO_Enter ||
+	    event->keyval == GDK_KP_Enter || event->keyval == GDK_space ||
+	    event->keyval == GDK_KP_Space)
+	{
+		show_menu (combo, 0, event->time);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static void
 set_shadow_type (GeditStatusComboBox *combo)
 {
@@ -297,9 +328,13 @@ gedit_status_combo_box_init (GeditStatusComboBox *self)
 	self->priv->menu = gtk_menu_new ();
 	g_object_ref_sink (self->priv->menu);
 
-	g_signal_connect (self->priv->button, 
-			  "button-press-event", 
-			  G_CALLBACK (button_press_event), 
+	g_signal_connect (self->priv->button,
+			  "button-press-event",
+			  G_CALLBACK (button_press_event),
+			  self);
+	g_signal_connect (self->priv->button,
+			  "key-press-event",
+			  G_CALLBACK (key_press_event),
 			  self);
 	g_signal_connect (self->priv->menu,
 			  "deactivate",
@@ -308,12 +343,22 @@ gedit_status_combo_box_init (GeditStatusComboBox *self)
 }
 
 /* public functions */
+
+/**
+ * gedit_status_combo_box_new:
+ * @label: (allow-none):
+ */
 GtkWidget *
 gedit_status_combo_box_new (const gchar *label)
 {
 	return g_object_new (GEDIT_TYPE_STATUS_COMBO_BOX, "label", label, NULL);
 }
 
+/**
+ * gedit_status_combo_box_set_label:
+ * @combo:
+ * @label: (allow-none):
+ */
 void
 gedit_status_combo_box_set_label (GeditStatusComboBox *combo, 
 				  const gchar         *label)
@@ -342,6 +387,12 @@ item_activated (GtkMenuItem         *item,
 	gedit_status_combo_box_set_item (combo, item);
 }
 
+/**
+ * gedit_status_combo_box_add_item:
+ * @combo:
+ * @item:
+ * @text: (allow-none):
+ */
 void
 gedit_status_combo_box_add_item (GeditStatusComboBox *combo,
 				 GtkMenuItem         *item,
@@ -367,6 +418,12 @@ gedit_status_combo_box_remove_item (GeditStatusComboBox *combo,
 			      GTK_WIDGET (item));
 }
 
+/**
+ * gedit_status_combo_box_get_items:
+ * @combo:
+ *
+ * Returns: (element-type Gtk.Widget) (transfer container):
+ */
 GList *
 gedit_status_combo_box_get_items (GeditStatusComboBox *combo)
 {
@@ -389,6 +446,12 @@ gedit_status_combo_box_get_item_text (GeditStatusComboBox *combo,
 	return ret;
 }
 
+/**
+ * gedit_status_combo_box_set_item_text:
+ * @combo:
+ * @item:
+ * @text: (allow-none):
+ */
 void 
 gedit_status_combo_box_set_item_text (GeditStatusComboBox *combo,
 				      GtkMenuItem	  *item,
