@@ -16,8 +16,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * $Id$
  */
 
 /*
@@ -33,30 +31,29 @@
 #include <string.h>
 #include <time.h>
 
-#include <gconf/gconf-client.h>
-
 #include "gedit-time-plugin.h"
-#include <gedit/gedit-help.h>
+#include <gedit/gedit-app.h>
 
 #include <glib/gi18n-lib.h>
 #include <glib.h>
-#include <gmodule.h>
 
 #include <gedit/gedit-debug.h>
 #include <gedit/gedit-utils.h>
+#include <gedit/gedit-window.h>
+#include <gedit/gedit-window-activatable.h>
+#include <libpeasui/peas-ui-configurable.h>
 
 #define GEDIT_TIME_PLUGIN_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), \
 					      GEDIT_TYPE_TIME_PLUGIN, \
 					      GeditTimePluginPrivate))
 
-#define WINDOW_DATA_KEY "GeditTimePluginWindowData"
 #define MENU_PATH "/MenuBar/EditMenu/EditOps_4"
 
-/* gconf keys */
-#define TIME_BASE_KEY		"/apps/gedit-2/plugins/time"
-#define PROMPT_TYPE_KEY		TIME_BASE_KEY "/prompt_type"
-#define SELECTED_FORMAT_KEY	TIME_BASE_KEY "/selected_format"
-#define CUSTOM_FORMAT_KEY	TIME_BASE_KEY "/custom_format"
+/* gsettings keys */
+#define TIME_BASE_SETTINGS	"org.gnome.gedit.plugins.time"
+#define PROMPT_TYPE_KEY		"prompt-type"
+#define SELECTED_FORMAT_KEY	"selected-format"
+#define CUSTOM_FORMAT_KEY	"custom-format"
 
 #define DEFAULT_CUSTOM_FORMAT "%d/%m/%Y %H:%M:%S"
 
@@ -105,6 +102,14 @@ enum
 	NUM_COLUMNS
 };
 
+typedef enum
+{
+	PROMPT_SELECTED_FORMAT = 0,	/* Popup dialog with list preselected */
+	PROMPT_CUSTOM_FORMAT,		/* Popup dialog with entry preselected */
+	USE_SELECTED_FORMAT,		/* Use selected format directly */
+	USE_CUSTOM_FORMAT		/* Use custom format directly */
+} GeditTimePluginPromptType;
+
 typedef struct _TimeConfigureDialog TimeConfigureDialog;
 
 struct _TimeConfigureDialog
@@ -113,16 +118,15 @@ struct _TimeConfigureDialog
 
 	GtkWidget *list;
 
-        /* Radio buttons to indicate what should be done */
-        GtkWidget *prompt;
-        GtkWidget *use_list;
-        GtkWidget *custom;
+	/* Radio buttons to indicate what should be done */
+	GtkWidget *prompt;
+	GtkWidget *use_list;
+	GtkWidget *custom;
 
 	GtkWidget *custom_entry;
 	GtkWidget *custom_format_example;
 
-	/* Info needed for the response handler */
-	GeditTimePlugin *plugin;
+	GSettings *settings;
 };
 
 typedef struct _ChooseFormatDialog ChooseFormatDialog;
@@ -133,46 +137,42 @@ struct _ChooseFormatDialog
 
 	GtkWidget *list;
 
-        /* Radio buttons to indicate what should be done */
-        GtkWidget *use_list;
-        GtkWidget *custom;
+	/* Radio buttons to indicate what should be done */
+	GtkWidget *use_list;
+	GtkWidget *custom;
 
-        GtkWidget *custom_entry;
+	GtkWidget *custom_entry;
 	GtkWidget *custom_format_example;
 
 	/* Info needed for the response handler */
 	GtkTextBuffer   *buffer;
-	GeditTimePlugin *plugin;
-};
 
-typedef enum
-{
-	PROMPT_SELECTED_FORMAT = 0,	/* Popup dialog with list preselected */
-	PROMPT_CUSTOM_FORMAT,		/* Popup dialog with entry preselected */
-	USE_SELECTED_FORMAT,		/* Use selected format directly */
-	USE_CUSTOM_FORMAT		/* Use custom format directly */
-} GeditTimePluginPromptType;
+	GSettings *settings;
+};
 
 struct _GeditTimePluginPrivate
 {
-	GConfClient *gconf_client;
-};
+	GSettings      *settings;
 
-GEDIT_PLUGIN_REGISTER_TYPE(GeditTimePlugin, gedit_time_plugin)
+	GeditWindow    *window;
 
-typedef struct
-{
 	GtkActionGroup *action_group;
 	guint           ui_id;
-} WindowData;
+};
 
-typedef struct
-{
-	GeditWindow     *window;
-	GeditTimePlugin *plugin;
-} ActionData;
+static void gedit_window_activatable_iface_init (GeditWindowActivatableInterface *iface);
+static void peas_ui_configurable_iface_init (PeasUIConfigurableInterface *iface);
 
-static void time_cb (GtkAction *action, ActionData *data);
+G_DEFINE_DYNAMIC_TYPE_EXTENDED (GeditTimePlugin,
+				gedit_time_plugin,
+				PEAS_TYPE_EXTENSION_BASE,
+				0,
+				G_IMPLEMENT_INTERFACE_DYNAMIC (GEDIT_TYPE_WINDOW_ACTIVATABLE,
+							       gedit_window_activatable_iface_init)
+				G_IMPLEMENT_INTERFACE_DYNAMIC (PEAS_UI_TYPE_CONFIGURABLE,
+							       peas_ui_configurable_iface_init))
+
+static void time_cb (GtkAction *action, GeditTimePlugin *plugin);
 
 static const GtkActionEntry action_entries[] =
 {
@@ -193,51 +193,44 @@ gedit_time_plugin_init (GeditTimePlugin *plugin)
 
 	plugin->priv = GEDIT_TIME_PLUGIN_GET_PRIVATE (plugin);
 
-	plugin->priv->gconf_client = gconf_client_get_default ();
-
-	gconf_client_add_dir (plugin->priv->gconf_client,
-			      TIME_BASE_KEY,
-			      GCONF_CLIENT_PRELOAD_ONELEVEL,
-			      NULL);
+	plugin->priv->settings = g_settings_new (TIME_BASE_SETTINGS);
 }
 
 static void
-gedit_time_plugin_finalize (GObject *object)
+gedit_time_plugin_dispose (GObject *object)
 {
 	GeditTimePlugin *plugin = GEDIT_TIME_PLUGIN (object);
 
-	gedit_debug_message (DEBUG_PLUGINS, "GeditTimePlugin finalizing");
+	gedit_debug_message (DEBUG_PLUGINS, "GeditTimePlugin disposing");
 
-	gconf_client_suggest_sync (plugin->priv->gconf_client, NULL);
+	if (plugin->priv->settings != NULL)
+	{
+		g_object_unref (plugin->priv->settings);
+		plugin->priv->settings = NULL;
+	}
 
-	g_object_unref (G_OBJECT (plugin->priv->gconf_client));
+	if (plugin->priv->action_group != NULL)
+	{
+		g_object_unref (plugin->priv->action_group);
+		plugin->priv->action_group = NULL;
+	}
 
-	G_OBJECT_CLASS (gedit_time_plugin_parent_class)->finalize (object);
+	G_OBJECT_CLASS (gedit_time_plugin_parent_class)->dispose (object);
 }
 
 static void
-free_window_data (WindowData *data)
-{
-	g_return_if_fail (data != NULL);
-
-	g_object_unref (data->action_group);
-	g_free (data);
-}
-
-static void
-update_ui_real (GeditWindow  *window,
-		WindowData   *data)
+update_ui (GeditTimePlugin *plugin)
 {
 	GeditView *view;
 	GtkAction *action;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	view = gedit_window_get_active_view (window);
+	view = gedit_window_get_active_view (plugin->priv->window);
 
 	gedit_debug_message (DEBUG_PLUGINS, "View: %p", view);
 
-	action = gtk_action_group_get_action (data->action_group,
+	action = gtk_action_group_get_action (plugin->priv->action_group,
 					      "InsertDateAndTime");
 	gtk_action_set_sensitive (action,
 				  (view != NULL) &&
@@ -245,146 +238,74 @@ update_ui_real (GeditWindow  *window,
 }
 
 static void
-impl_activate (GeditPlugin *plugin,
-	       GeditWindow *window)
+gedit_time_plugin_activate (GeditWindowActivatable *activatable,
+			    GeditWindow            *window)
 {
+	GeditTimePluginPrivate *priv;
 	GtkUIManager *manager;
-	WindowData *data;
-	ActionData *action_data;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	data = g_new (WindowData, 1);
-	action_data = g_new (ActionData, 1);
-
-	action_data->plugin = GEDIT_TIME_PLUGIN (plugin);
-	action_data->window = window;
+	priv = GEDIT_TIME_PLUGIN (activatable)->priv;
+	priv->window = window;
 
 	manager = gedit_window_get_ui_manager (window);
 
-	data->action_group = gtk_action_group_new ("GeditTimePluginActions");
-	gtk_action_group_set_translation_domain (data->action_group,
+	priv->action_group = gtk_action_group_new ("GeditTimePluginActions");
+	gtk_action_group_set_translation_domain (priv->action_group,
 						 GETTEXT_PACKAGE);
-	gtk_action_group_add_actions_full (data->action_group,
-				      	   action_entries,
-				      	   G_N_ELEMENTS (action_entries),
-				      	   action_data,
-				      	   (GDestroyNotify) g_free);
+	gtk_action_group_add_actions (priv->action_group,
+				      action_entries,
+				      G_N_ELEMENTS (action_entries),
+				      activatable);
 
-	gtk_ui_manager_insert_action_group (manager, data->action_group, -1);
+	gtk_ui_manager_insert_action_group (manager, priv->action_group, -1);
 
-	data->ui_id = gtk_ui_manager_new_merge_id (manager);
-
-	g_object_set_data_full (G_OBJECT (window),
-				WINDOW_DATA_KEY,
-				data,
-				(GDestroyNotify) free_window_data);
+	priv->ui_id = gtk_ui_manager_new_merge_id (manager);
 
 	gtk_ui_manager_add_ui (manager,
-			       data->ui_id,
+			       priv->ui_id,
 			       MENU_PATH,
 			       "InsertDateAndTime",
 			       "InsertDateAndTime",
 			       GTK_UI_MANAGER_MENUITEM,
 			       FALSE);
 
-	update_ui_real (window, data);
+	update_ui (GEDIT_TIME_PLUGIN (activatable));
 }
 
 static void
-impl_deactivate	(GeditPlugin *plugin,
-		 GeditWindow *window)
+gedit_time_plugin_deactivate (GeditWindowActivatable *activatable,
+			      GeditWindow            *window)
 {
+	GeditTimePluginPrivate *priv;
 	GtkUIManager *manager;
-	WindowData *data;
 
 	gedit_debug (DEBUG_PLUGINS);
 
+	priv = GEDIT_TIME_PLUGIN (activatable)->priv;
+
 	manager = gedit_window_get_ui_manager (window);
 
-	data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
-	g_return_if_fail (data != NULL);
-
-	gtk_ui_manager_remove_ui (manager, data->ui_id);
-	gtk_ui_manager_remove_action_group (manager, data->action_group);
-
-	g_object_set_data (G_OBJECT (window), WINDOW_DATA_KEY, NULL);
+	gtk_ui_manager_remove_ui (manager, priv->ui_id);
+	gtk_ui_manager_remove_action_group (manager, priv->action_group);
 }
 
 static void
-impl_update_ui	(GeditPlugin *plugin,
-		 GeditWindow *window)
+gedit_time_plugin_update_state (GeditWindowActivatable *activatable,
+				GeditWindow            *window)
 {
-	WindowData   *data;
-
 	gedit_debug (DEBUG_PLUGINS);
 
-	data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
-	g_return_if_fail (data != NULL);
-
-	update_ui_real (window, data);
+	update_ui (GEDIT_TIME_PLUGIN (activatable));
 }
 
 /* whether we should prompt the user or use the specified format */
 static GeditTimePluginPromptType
 get_prompt_type (GeditTimePlugin *plugin)
 {
-	gchar *prompt_type;
-	GeditTimePluginPromptType res;
-
-	prompt_type = gconf_client_get_string (plugin->priv->gconf_client,
-			        	       PROMPT_TYPE_KEY,
-					       NULL);
-
-	if (prompt_type == NULL)
-		return PROMPT_SELECTED_FORMAT;
-
-	if (strcmp (prompt_type, "USE_SELECTED_FORMAT") == 0)
-		res = USE_SELECTED_FORMAT;
-	else if (strcmp (prompt_type, "USE_CUSTOM_FORMAT") == 0)
-		res = USE_CUSTOM_FORMAT;
-	else if (strcmp (prompt_type, "PROMPT_CUSTOM_FORMAT") == 0)
-		res = PROMPT_CUSTOM_FORMAT;
-	else
-		res = PROMPT_SELECTED_FORMAT;
-
-	g_free (prompt_type);
-
-	return res;
-}
-
-static void
-set_prompt_type (GeditTimePlugin           *plugin,
-		 GeditTimePluginPromptType  prompt_type)
-{
-	const gchar * str;
-
-	if (!gconf_client_key_is_writable (plugin->priv->gconf_client,
-					   PROMPT_TYPE_KEY,
-					   NULL))
-	{
-		return;
-	}
-
-	switch (prompt_type)
-	{
-		case USE_SELECTED_FORMAT:
-			str = "USE_SELECTED_FORMAT";
-			break;
-		case USE_CUSTOM_FORMAT:
-			str = "USE_CUSTOM_FORMAT";
-			break;
-		case PROMPT_CUSTOM_FORMAT:
-			str = "PROMPT_CUSTOM_FORMAT";
-			break;
-		default:
-			str = "PROMPT_SELECTED_FORMAT";
-	}
-
-	gconf_client_set_string (plugin->priv->gconf_client,
-				 PROMPT_TYPE_KEY,
-		       		 str,
-		       		 NULL);
+	return g_settings_get_enum (plugin->priv->settings,
+				    PROMPT_TYPE_KEY);
 }
 
 /* The selected format in the list */
@@ -393,70 +314,32 @@ get_selected_format (GeditTimePlugin *plugin)
 {
 	gchar *sel_format;
 
-	sel_format = gconf_client_get_string (plugin->priv->gconf_client,
-					      SELECTED_FORMAT_KEY,
-					      NULL);
+	sel_format = g_settings_get_string (plugin->priv->settings,
+					    SELECTED_FORMAT_KEY);
 
 	return sel_format ? sel_format : g_strdup (formats [0]);
 }
 
-static void
-set_selected_format (GeditTimePlugin *plugin,
-		     const gchar     *format)
-{
-	g_return_if_fail (format != NULL);
-
-	if (!gconf_client_key_is_writable (plugin->priv->gconf_client,
-					   SELECTED_FORMAT_KEY,
-					   NULL))
-	{
-		return;
-	}
-
-	gconf_client_set_string (plugin->priv->gconf_client,
-				 SELECTED_FORMAT_KEY,
-		       		 format,
-		       		 NULL);
-}
-
 /* the custom format in the entry */
 static gchar *
 get_custom_format (GeditTimePlugin *plugin)
 {
 	gchar *format;
 
-	format = gconf_client_get_string (plugin->priv->gconf_client,
-					  CUSTOM_FORMAT_KEY,
-					  NULL);
+	format = g_settings_get_string (plugin->priv->settings,
+					CUSTOM_FORMAT_KEY);
 
 	return format ? format : g_strdup (DEFAULT_CUSTOM_FORMAT);
 }
 
-static void
-set_custom_format (GeditTimePlugin *plugin,
-		   const gchar     *format)
-{
-	g_return_if_fail (format != NULL);
-
-	if (!gconf_client_key_is_writable (plugin->priv->gconf_client,
-					   CUSTOM_FORMAT_KEY,
-					   NULL))
-		return;
-
-	gconf_client_set_string (plugin->priv->gconf_client,
-				 CUSTOM_FORMAT_KEY,
-		       		 format,
-		       		 NULL);
-}
-
 static gchar *
 get_time (const gchar* format)
 {
-  	gchar *out = NULL;
+	gchar *out = NULL;
 	gchar *out_utf8 = NULL;
-  	time_t clock;
-  	struct tm *now;
-  	size_t out_length = 0;
+	time_t clock;
+	struct tm *now;
+	size_t out_length = 0;
 	gchar *locale_format;
 
 	gedit_debug (DEBUG_PLUGINS);
@@ -470,15 +353,15 @@ get_time (const gchar* format)
 	if (locale_format == NULL)
 		return g_strdup (" ");
 
-  	clock = time (NULL);
-  	now = localtime (&clock);
+	clock = time (NULL);
+	now = localtime (&clock);
 
 	do
 	{
 		out_length += 255;
 		out = g_realloc (out, out_length);
 	}
-  	while (strftime (out, out_length, locale_format, now) == 0);
+	while (strftime (out, out_length, locale_format, now) == 0);
 
 	g_free (locale_format);
 
@@ -495,15 +378,30 @@ get_time (const gchar* format)
 			out_utf8 = g_strdup (" ");
 	}
 
-  	return out_utf8;
+	return out_utf8;
 }
 
 static void
-dialog_destroyed (GtkObject *obj, gpointer dialog_pointer)
+configure_dialog_destroyed (GtkObject *obj,
+			    gpointer   dialog_pointer)
+{
+	TimeConfigureDialog *dialog = (TimeConfigureDialog *)dialog_pointer;
+
+	gedit_debug (DEBUG_PLUGINS);
+
+	g_object_unref (dialog->settings);
+	g_slice_free (TimeConfigureDialog, dialog_pointer);
+
+	gedit_debug_message (DEBUG_PLUGINS, "END");
+}
+
+static void
+choose_format_dialog_destroyed (GtkObject *obj,
+				gpointer   dialog_pointer)
 {
 	gedit_debug (DEBUG_PLUGINS);
 
-	g_free (dialog_pointer);
+	g_slice_free (ChooseFormatDialog, dialog_pointer);
 
 	gedit_debug_message (DEBUG_PLUGINS, "END");
 }
@@ -656,7 +554,7 @@ updated_custom_format_example (GtkEntry *format_entry,
 }
 
 static void
-choose_format_dialog_button_toggled (GtkToggleButton *button,
+choose_format_dialog_button_toggled (GtkToggleButton    *button,
 				     ChooseFormatDialog *dialog)
 {
 	gedit_debug (DEBUG_PLUGINS);
@@ -666,22 +564,22 @@ choose_format_dialog_button_toggled (GtkToggleButton *button,
 		gtk_widget_set_sensitive (dialog->list, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_entry, TRUE);
 		gtk_widget_set_sensitive (dialog->custom_format_example, TRUE);
-
-		return;
 	}
-
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->use_list)))
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->use_list)))
 	{
 		gtk_widget_set_sensitive (dialog->list, TRUE);
 		gtk_widget_set_sensitive (dialog->custom_entry, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_format_example, FALSE);
-
-		return;
+	}
+	else
+	{
+		g_return_if_reached ();
 	}
 }
 
 static void
-configure_dialog_button_toggled (GtkToggleButton *button, TimeConfigureDialog *dialog)
+configure_dialog_button_toggled (GtkToggleButton     *button,
+				 TimeConfigureDialog *dialog)
 {
 	gedit_debug (DEBUG_PLUGINS);
 
@@ -690,26 +588,22 @@ configure_dialog_button_toggled (GtkToggleButton *button, TimeConfigureDialog *d
 		gtk_widget_set_sensitive (dialog->list, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_entry, TRUE);
 		gtk_widget_set_sensitive (dialog->custom_format_example, TRUE);
-
-		return;
 	}
-
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->use_list)))
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->use_list)))
 	{
 		gtk_widget_set_sensitive (dialog->list, TRUE);
 		gtk_widget_set_sensitive (dialog->custom_entry, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_format_example, FALSE);
-
-		return;
 	}
-
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->prompt)))
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->prompt)))
 	{
 		gtk_widget_set_sensitive (dialog->list, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_entry, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_format_example, FALSE);
-
-		return;
+	}
+	else
+	{
+		g_return_if_reached ();
 	}
 }
 
@@ -730,13 +624,13 @@ get_format_from_list (GtkWidget *listview)
 
 	if (gtk_tree_selection_get_selected (selection, NULL, &iter))
 	{
-	        gint selected_value;
+		gint selected_value;
 
 		gtk_tree_model_get (model, &iter, COLUMN_INDEX, &selected_value, -1);
 
 		gedit_debug_message (DEBUG_PLUGINS, "Sel value: %d", selected_value);
 
-	        return selected_value;
+		return selected_value;
 	}
 
 	g_return_val_if_reached (0);
@@ -761,7 +655,8 @@ get_configure_dialog (GeditTimePlugin *plugin)
 	
 	gedit_debug (DEBUG_PLUGINS);
 
-	dialog = g_new0 (TimeConfigureDialog, 1);
+	dialog = g_slice_new (TimeConfigureDialog);
+	dialog->settings = g_object_ref (plugin->priv->settings);
 
 	dialog->dialog = gtk_dialog_new_with_buttons (_("Configure insert date/time plugin..."),
 						      NULL,
@@ -784,7 +679,7 @@ get_configure_dialog (GeditTimePlugin *plugin)
 
 	g_return_val_if_fail (dialog->dialog != NULL, NULL);
 
-	data_dir = gedit_plugin_get_data_dir (GEDIT_PLUGIN (plugin));
+	data_dir = peas_extension_base_get_data_dir (PEAS_EXTENSION_BASE (plugin));
 	ui_file = g_build_filename (data_dir, "gedit-time-setup-dialog.ui", NULL);
 	ret = gedit_utils_get_ui_objects (ui_file,
 					  root_objects,
@@ -824,36 +719,36 @@ get_configure_dialog (GeditTimePlugin *plugin)
 	prompt_type = get_prompt_type (plugin);
 
 	cf = get_custom_format (plugin);
-     	gtk_entry_set_text (GTK_ENTRY(dialog->custom_entry), cf);
-       	g_free (cf);
+	gtk_entry_set_text (GTK_ENTRY(dialog->custom_entry), cf);
+	g_free (cf);
 
-        if (prompt_type == USE_CUSTOM_FORMAT)
-        {
-	        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->custom), TRUE);
+	if (prompt_type == USE_CUSTOM_FORMAT)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->custom), TRUE);
 
 		gtk_widget_set_sensitive (dialog->list, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_entry, TRUE);
 		gtk_widget_set_sensitive (dialog->custom_format_example, TRUE);
-        }
-        else if (prompt_type == USE_SELECTED_FORMAT)
-        {
-	        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->use_list), TRUE);
+	}
+	else if (prompt_type == USE_SELECTED_FORMAT)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->use_list), TRUE);
 
 		gtk_widget_set_sensitive (dialog->list, TRUE);
 		gtk_widget_set_sensitive (dialog->custom_entry, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_format_example, FALSE);
-        }
-        else
-        {
-	        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->prompt), TRUE);
+	}
+	else
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->prompt), TRUE);
 
 		gtk_widget_set_sensitive (dialog->list, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_entry, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_format_example, FALSE);
-        }
+	}
 
 	updated_custom_format_example (GTK_ENTRY (dialog->custom_entry),
-			GTK_LABEL (dialog->custom_format_example));
+				       GTK_LABEL (dialog->custom_format_example));
 
 	/* setup a window of a sane size. */
 	gtk_widget_set_size_request (GTK_WIDGET (viewport), 10, 200);
@@ -870,7 +765,7 @@ get_configure_dialog (GeditTimePlugin *plugin)
 			  "toggled",
 			  G_CALLBACK (configure_dialog_button_toggled),
 			  dialog);
-   	g_signal_connect (dialog->prompt,
+	g_signal_connect (dialog->prompt,
 			  "toggled",
 			  G_CALLBACK (configure_dialog_button_toggled),
 			  dialog);
@@ -880,7 +775,7 @@ get_configure_dialog (GeditTimePlugin *plugin)
 			  dialog);
 	g_signal_connect (dialog->dialog,
 			  "destroy",
-			  G_CALLBACK (dialog_destroyed),
+			  G_CALLBACK (configure_dialog_destroyed),
 			  dialog);
 	g_signal_connect (dialog->custom_entry,
 			  "changed",
@@ -916,8 +811,12 @@ choose_format_dialog_row_activated (GtkTreeView        *list,
 	sel_format = get_format_from_list (dialog->list);
 	the_time = get_time (formats[sel_format]);
 
-	set_prompt_type (dialog->plugin, PROMPT_SELECTED_FORMAT);
-	set_selected_format (dialog->plugin, formats[sel_format]);
+	g_settings_set_enum (dialog->settings,
+			     PROMPT_TYPE_KEY,
+			     PROMPT_SELECTED_FORMAT);
+	g_settings_set_string (dialog->settings,
+			       SELECTED_FORMAT_KEY,
+			       formats[sel_format]);
 
 	g_return_if_fail (the_time != NULL);
 
@@ -942,9 +841,10 @@ get_choose_format_dialog (GtkWindow                 *parent,
 	if (parent != NULL)
 		wg = gtk_window_get_group (parent);
 
-	dialog = g_new0 (ChooseFormatDialog, 1);
+	dialog = g_slice_new (ChooseFormatDialog);
+	dialog->settings = plugin->priv->settings;
 
-	data_dir = gedit_plugin_get_data_dir (GEDIT_PLUGIN (plugin));
+	data_dir = peas_extension_base_get_data_dir (PEAS_EXTENSION_BASE (plugin));
 	ui_file = g_build_filename (data_dir, "gedit-time-dialog.ui", NULL);
 	ret = gedit_utils_get_ui_objects (ui_file,
 					  NULL,
@@ -991,7 +891,7 @@ get_choose_format_dialog (GtkWindow                 *parent,
 	}
 
 	gtk_window_group_add_window (wg,
-			     	     GTK_WINDOW (dialog->dialog));
+				     GTK_WINDOW (dialog->dialog));
 	gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog), parent);
 	gtk_window_set_modal (GTK_WINDOW (dialog->dialog), TRUE);
 
@@ -1000,7 +900,7 @@ get_choose_format_dialog (GtkWindow                 *parent,
 	g_free (sf);
 
 	cf = get_custom_format (plugin);
-     	gtk_entry_set_text (GTK_ENTRY(dialog->custom_entry), cf);
+	gtk_entry_set_text (GTK_ENTRY(dialog->custom_entry), cf);
 	g_free (cf);
 
 	updated_custom_format_example (GTK_ENTRY (dialog->custom_entry),
@@ -1008,7 +908,7 @@ get_choose_format_dialog (GtkWindow                 *parent,
 
 	if (prompt_type == PROMPT_CUSTOM_FORMAT)
 	{
-        	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->custom), TRUE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->custom), TRUE);
 
 		gtk_widget_set_sensitive (dialog->list, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_entry, TRUE);
@@ -1043,7 +943,7 @@ get_choose_format_dialog (GtkWindow                 *parent,
 			  dialog);
 	g_signal_connect (dialog->dialog,
 			  "destroy",
-			  G_CALLBACK (dialog_destroyed),
+			  G_CALLBACK (choose_format_dialog_destroyed),
 			  dialog);
 	g_signal_connect (dialog->custom_entry,
 			  "changed",
@@ -1069,9 +969,10 @@ choose_format_dialog_response_cb (GtkWidget          *widget,
 		case GTK_RESPONSE_HELP:
 		{
 			gedit_debug_message (DEBUG_PLUGINS, "GTK_RESPONSE_HELP");
-			gedit_help_display (GTK_WINDOW (widget),
-					    NULL,
-					    "gedit-insert-date-time-plugin");
+			gedit_app_show_help (gedit_app_get_default (),
+					     GTK_WINDOW (widget),
+					     NULL,
+					     "gedit-insert-date-time-plugin");
 			break;
 		}
 		case GTK_RESPONSE_OK:
@@ -1088,8 +989,12 @@ choose_format_dialog_response_cb (GtkWidget          *widget,
 				sel_format = get_format_from_list (dialog->list);
 				the_time = get_time (formats[sel_format]);
 
-				set_prompt_type (dialog->plugin, PROMPT_SELECTED_FORMAT);
-				set_selected_format (dialog->plugin, formats[sel_format]);
+				g_settings_set_enum (dialog->settings,
+						     PROMPT_TYPE_KEY,
+						     PROMPT_SELECTED_FORMAT);
+				g_settings_set_string (dialog->settings,
+						       SELECTED_FORMAT_KEY,
+						       formats[sel_format]);
 			}
 			else
 			{
@@ -1098,8 +1003,12 @@ choose_format_dialog_response_cb (GtkWidget          *widget,
 				format = gtk_entry_get_text (GTK_ENTRY (dialog->custom_entry));
 				the_time = get_time (format);
 
-				set_prompt_type (dialog->plugin, PROMPT_CUSTOM_FORMAT);
-				set_custom_format (dialog->plugin, format);
+				g_settings_set_enum (dialog->settings,
+						     PROMPT_TYPE_KEY,
+						     PROMPT_CUSTOM_FORMAT);
+				g_settings_set_string (dialog->settings,
+						       CUSTOM_FORMAT_KEY,
+						       format);
 			}
 
 			g_return_if_fail (the_time != NULL);
@@ -1117,43 +1026,46 @@ choose_format_dialog_response_cb (GtkWidget          *widget,
 }
 
 static void
-time_cb (GtkAction  *action,
-	 ActionData *data)
+time_cb (GtkAction       *action,
+	 GeditTimePlugin *plugin)
 {
+	GeditTimePluginPrivate *priv;
 	GtkTextBuffer *buffer;
-	gchar *the_time = NULL;
 	GeditTimePluginPromptType prompt_type;
+	gchar *the_time = NULL;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	buffer = GTK_TEXT_BUFFER (gedit_window_get_active_document (data->window));
+	priv = plugin->priv;
+
+	buffer = GTK_TEXT_BUFFER (gedit_window_get_active_document (priv->window));
 	g_return_if_fail (buffer != NULL);
 
-	prompt_type = get_prompt_type (data->plugin);
+	prompt_type = get_prompt_type (plugin);
 
-        if (prompt_type == USE_CUSTOM_FORMAT)
-        {
-		gchar *cf = get_custom_format (data->plugin);
-	        the_time = get_time (cf);
+	if (prompt_type == USE_CUSTOM_FORMAT)
+	{
+		gchar *cf = get_custom_format (plugin);
+		the_time = get_time (cf);
 		g_free (cf);
 	}
-        else if (prompt_type == USE_SELECTED_FORMAT)
-        {
-		gchar *sf = get_selected_format (data->plugin);
-	        the_time = get_time (sf);
+	else if (prompt_type == USE_SELECTED_FORMAT)
+	{
+		gchar *sf = get_selected_format (plugin);
+		the_time = get_time (sf);
 		g_free (sf);
 	}
-        else
-        {
+	else
+	{
 		ChooseFormatDialog *dialog;
 
-		dialog = get_choose_format_dialog (GTK_WINDOW (data->window),
+		dialog = get_choose_format_dialog (GTK_WINDOW (priv->window),
 						   prompt_type,
-						   data->plugin);
+						   plugin);
 		if (dialog != NULL)
 		{
 			dialog->buffer = buffer;
-			dialog->plugin = data->plugin;
+			dialog->settings = plugin->priv->settings;
 
 			g_signal_connect (dialog->dialog,
 					  "response",
@@ -1176,6 +1088,7 @@ time_cb (GtkAction  *action,
 static void
 ok_button_pressed (TimeConfigureDialog *dialog)
 {
+	GeditTimePluginPromptType prompt_type;
 	gint sel_format;
 	const gchar *custom_format;
 
@@ -1187,20 +1100,30 @@ ok_button_pressed (TimeConfigureDialog *dialog)
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->custom)))
 	{
-		set_prompt_type (dialog->plugin, USE_CUSTOM_FORMAT);
-		set_custom_format (dialog->plugin, custom_format);
+		prompt_type = USE_CUSTOM_FORMAT;
+
+		g_settings_set_string (dialog->settings,
+				       CUSTOM_FORMAT_KEY,
+				       custom_format);
 	}
 	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->use_list)))
 	{
-		set_prompt_type (dialog->plugin, USE_SELECTED_FORMAT);
-		set_selected_format (dialog->plugin, formats [sel_format]);
+		prompt_type = USE_SELECTED_FORMAT;
+
+		g_settings_set_string (dialog->settings,
+				       SELECTED_FORMAT_KEY,
+				       formats[sel_format]);
 	}
 	else
 	{
 		/* Default to prompt the user with the list selected */
-		set_prompt_type (dialog->plugin, PROMPT_SELECTED_FORMAT);
+		prompt_type = PROMPT_SELECTED_FORMAT;
 	}
 
+	g_settings_set_enum (dialog->settings,
+			     PROMPT_TYPE_KEY,
+			     prompt_type);
+
 	gedit_debug_message (DEBUG_PLUGINS, "Sel: %d", sel_format);
 }
 
@@ -1215,9 +1138,10 @@ configure_dialog_response_cb (GtkWidget           *widget,
 		{
 			gedit_debug_message (DEBUG_PLUGINS, "GTK_RESPONSE_HELP");
 
-			gedit_help_display (GTK_WINDOW (dialog),
-					    NULL,
-					    "gedit-date-time-configure");
+			gedit_app_show_help (gedit_app_get_default (),
+					     GTK_WINDOW (widget),
+					     NULL,
+					     "gedit-date-time-configure");
 			break;
 		}
 		case GTK_RESPONSE_OK:
@@ -1226,47 +1150,75 @@ configure_dialog_response_cb (GtkWidget           *widget,
 
 			ok_button_pressed (dialog);
 
-			gtk_widget_destroy (dialog->dialog);
+			gtk_widget_destroy (widget);
 			break;
 		}
 		case GTK_RESPONSE_CANCEL:
 		{
 			gedit_debug_message (DEBUG_PLUGINS, "GTK_RESPONSE_CANCEL");
-			gtk_widget_destroy (dialog->dialog);
+			gtk_widget_destroy (widget);
 		}
 	}
 }
 
-static GtkWidget *
-impl_create_configure_dialog (GeditPlugin *plugin)
+static gboolean
+gedit_time_plugin_create_configure_dialog (PeasUIConfigurable *configurable,
+					   GtkWidget         **dialog)
 {
-	TimeConfigureDialog *dialog;
+	TimeConfigureDialog *dlg;
 
-	dialog = get_configure_dialog (GEDIT_TIME_PLUGIN (plugin));
+	dlg = get_configure_dialog (GEDIT_TIME_PLUGIN (configurable));
 
-	dialog->plugin = GEDIT_TIME_PLUGIN (plugin);
-
-	g_signal_connect (dialog->dialog,
+	g_signal_connect (dlg->dialog,
 			  "response",
 			  G_CALLBACK (configure_dialog_response_cb),
-			  dialog);
+			  dlg);
 
-	return GTK_WIDGET (dialog->dialog);
+	*dialog = GTK_WIDGET (dlg->dialog);
+
+	return TRUE;
 }
 
 static void
 gedit_time_plugin_class_init (GeditTimePluginClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GeditPluginClass *plugin_class = GEDIT_PLUGIN_CLASS (klass);
 
-	object_class->finalize = gedit_time_plugin_finalize;
-
-	plugin_class->activate = impl_activate;
-	plugin_class->deactivate = impl_deactivate;
-	plugin_class->update_ui = impl_update_ui;
-
-	plugin_class->create_configure_dialog = impl_create_configure_dialog;
+	object_class->dispose = gedit_time_plugin_dispose;
 
 	g_type_class_add_private (object_class, sizeof (GeditTimePluginPrivate));
 }
+
+static void
+gedit_time_plugin_class_finalize (GeditTimePluginClass *klass)
+{
+}
+
+static void
+peas_ui_configurable_iface_init (PeasUIConfigurableInterface *iface)
+{
+	iface->create_configure_dialog = gedit_time_plugin_create_configure_dialog;
+}
+
+static void
+gedit_window_activatable_iface_init (GeditWindowActivatableInterface *iface)
+{
+	iface->activate = gedit_time_plugin_activate;
+	iface->deactivate = gedit_time_plugin_deactivate;
+	iface->update_state = gedit_time_plugin_update_state;
+}
+
+G_MODULE_EXPORT void
+peas_register_types (PeasObjectModule *module)
+{
+	gedit_time_plugin_register_type (G_TYPE_MODULE (module));
+
+	peas_object_module_register_extension_type (module,
+						    GEDIT_TYPE_WINDOW_ACTIVATABLE,
+						    GEDIT_TYPE_TIME_PLUGIN);
+	peas_object_module_register_extension_type (module,
+						    PEAS_UI_TYPE_CONFIGURABLE,
+						    GEDIT_TYPE_TIME_PLUGIN);
+}
+
+/* ex:ts=8:noet: */
